@@ -1,4 +1,8 @@
+from json import JSONDecodeError
+
 from . import __version__, __file__ as module_path
+
+from .movement_manager import MovementManager
 from .request import StationRESTServerThread, DEFAULT_REST_KWARGS
 from .utils import ProtocolContextLoggingHandler, LocalWebServerLogger
 from .lights import Button, BlinkingLightHTTP, BlinkingLight
@@ -89,6 +93,7 @@ class Station(metaclass=StationMeta):
         language: str = "ENG",
         metadata: Optional[dict] = None,
         num_samples: int = 96,
+        protocol_exception_filepath: Optional[str] = '/var/lib/jupyter/notebooks/outputs/error_{}.log',
         rest_server_kwargs: dict = DEFAULT_REST_KWARGS,
         samples_per_col: int = 8,
         skip_delay: bool = False,
@@ -118,7 +123,9 @@ class Station(metaclass=StationMeta):
         self._log_lws_endpoint = log_lws_endpoint
         self._logger = logger
         self.metadata = metadata
+        self._mov_manager = None
         self._num_samples = num_samples
+        self._protocol_exception_filepath = protocol_exception_filepath.format(time.strftime("%Y_%m_%d__%H_%M_%S"))
         self._rest_server_kwargs = rest_server_kwargs
         self._samples_per_col = samples_per_col
         self._start_at = start_at
@@ -169,6 +176,11 @@ class Station(metaclass=StationMeta):
             self._run_stage = True
         self.logger.info("[{}] Stage: {}".format("x" if self._run_stage else " ", self.stage))
         return self._run_stage
+
+    def assert_run_stage_has_been_executed(self):
+        ''' If a _start_at_ value was requested, check it at least a corresponding stage was found in protocol.'''
+        if not self._run_stage:
+            raise Exception("Stage '{}' not found.".format(self._start_at))
     
     @property
     def logger(self) -> logging.getLoggerClass():
@@ -192,7 +204,11 @@ class Station(metaclass=StationMeta):
             self._lws_logger = LocalWebServerLogger(self._log_lws_ip, self._log_lws_endpoint)
             if self._simulation_log_lws or not self._ctx.is_simulating():
                 self._ctx.broker.subscribe(opentrons.commands.types.COMMAND, self._lws_logger)
-    
+
+    def log_protocol_exception(self, e: Exception):
+        with open(self._protocol_exception_filepath, 'w') as f:
+            f.write(json.dumps({"error": "{}".format(e)}))
+
     @property
     def logger_name(self) -> str:
         return self.__class__.__name__
@@ -256,10 +272,15 @@ class Station(metaclass=StationMeta):
         data_tip_status = {}
         if self._tip_track:
             self.logger.info(self.msg_format("tip info log", self._tip_log_filepath))
-            if os.path.isfile(self._tip_log_filepath):
-                with open(self._tip_log_filepath) as json_file:
-                    file = json.load(json_file)
-                    data_tip_status: dict = file.get("tip_status", {})
+            # Checking  with os.stat if the file is empty, so we don't have a JSONDecodeError
+            if os.path.isfile(self._tip_log_filepath) and os.stat(self._tip_log_filepath).st_size > 0:
+                try:
+                    with open(self._tip_log_filepath) as json_file:
+                        file = json.load(json_file)
+                        data_tip_status: dict = file.get("tip_status", {})
+                except JSONDecodeError as e:
+                    self.logger.error("Error during tip log read: {}".format(e))
+                    raise Exception("Tip log malformed. Please reset it.")
         else:
             self.logger.debug("not using tip log file")
         
@@ -347,6 +368,8 @@ class Station(metaclass=StationMeta):
         level: int = logging.INFO,
         pause: bool = True,
     ):
+        # Note: changed default homing method to 'move_to_home' Agos, 2021-09-13
+
         self.status = "pause"
         old_color = self._button.color
         self._button.color = color
@@ -354,7 +377,7 @@ class Station(metaclass=StationMeta):
             self.msg = msg
             self.logger.log(level, self.msg)
         if home:
-            self._ctx.home()
+            self._mov_manager.move_to_home()
         if blink and not self._ctx.is_simulating():
             lt = (BlinkingLightHTTP if self._dummy_lights else BlinkingLight)(self._ctx, t=blink_period/2)
             lt.start()
@@ -369,7 +392,9 @@ class Station(metaclass=StationMeta):
         self._button.color = old_color
         self.status = "running"
         self.msg = ""
-    
+
+
+
     def dual_pause(self, msg: str, cols: Tuple[str, str] = ('red', 'yellow'), between: Optional[Callable] = None, home: Tuple[bool, bool] = (True, False)):
         msg = self.get_msg(msg)
         self._msg = "{}.\n{}".format(msg, self.get_msg("stop blink"))
@@ -402,6 +427,7 @@ class Station(metaclass=StationMeta):
     def run(self, ctx: ProtocolContext):
         self.status = "running"
         self._ctx = ctx
+        self._mov_manager = MovementManager(self._ctx)
         self._button = (Button.dummy if self._dummy_lights else Button)(self._ctx, 'blue')
         if self._simulation_log_lws or not self._ctx.is_simulating():
             self._request = StationRESTServerThread(ctx, station=self, **self._rest_server_kwargs)
@@ -416,18 +442,26 @@ class Station(metaclass=StationMeta):
         self.logger.info(self.msg_format("protocol description"))
         self.logger.info(self.msg_format("num samples", self._num_samples))
         self.logger.info(self.msg_format("version", __version__))
-        
-        self.load_labware()
-        self.load_instruments()
-        self.setup_tip_log()
-        self._button.color = 'white'
-        self.msg = ""
 
-        if self._debug_mode:
-            self.dual_pause("debug mode", home=(False, False))
-        
         try:
+            self.load_labware()
+            self.load_instruments()
+            self.setup_tip_log()
+            self._button.color = 'white'
+            self.msg = ""
+
+            if self._debug_mode:
+                self.dual_pause("debug mode", home=(False, False))
+
             self.body()
+
+            self.assert_run_stage_has_been_executed()
+
+        except Exception as e:
+            self.logger.error("Exception occurred during protocol: {}".format(e))
+            if not self._ctx.is_simulating():
+                self.log_protocol_exception(e)
+            raise e     # Propagate the exception
         finally:
             self.status = "finished"
             if not self._ctx.is_simulating():
